unit FieldPaint;
interface
uses Classes,Graphics,Field,JPEG,ScaleBMP;

const MAXCOLORS=9;

type TFieldColorArray=array[0..MAXCOLORS]of TColor;

const DefaultFieldColors:TFieldColorArray=($302010,$0000C0,$00C000,$C00000,
                                           $C0C000,$C000C0,$00C0C0,$C0C0C0,
                                           $ABCDEF,$FEDCBA);
var FieldColors:TFieldColorArray;

var FieldBitmaps:array[0..MAXCOLORS]of TBitmap;
    UseBitmaps:boolean;

function LoadFieldBitmaps(const FileName:string):integer;
function LoadBackgroundImage(const FileName:string; IWidth,IHeight:integer):boolean;
procedure DrawField(Field:TField; Canvas:TCanvas; BWidth,BHeight,MarkOffX,MarkOffY:integer);

var Cache:TBitmap;

implementation

procedure DrawField(Field:TField; Canvas:TCanvas; BWidth,BHeight,MarkOffX,MarkOffY:integer);
var X,Y,BX,BY,F,G:integer;
begin
  BX:=Field.Width*BWidth; BY:=Field.Height*BHeight;
  if not(Assigned(Cache)) OR (Cache.Width<>BX) OR (Cache.Height<>BY) then begin
    if Assigned(Cache) then Cache.Free;
    Cache:=TBitmap.Create;
    with Cache do begin
      PixelFormat:=pfDevice;
      Width:=BX; Height:=BY;
    end;
  end;
  with Cache.Canvas,Field do begin
    if Assigned(FieldBitmaps[0]) then
      Draw(0,0,FieldBitmaps[0])
    else begin
      Brush.Color:=FieldColors[0];
      FillRect(Rect(0,0,BX,BY));
    end;
    BY:=0; for Y:=0 to (Height-1) do begin
      BX:=0; for X:=0 to (Width-1) do begin
        F:=Field[X,Y];
        if (F AND UNMARK)<>0 then
          if UseBitmaps then begin
            if (F AND MARKED)<>0
              then Draw(BX-MarkOffX,BY+MarkOffY,FieldBitmaps[F AND UNMARK])
              else Draw(BX,BY,FieldBitmaps[F AND UNMARK]);
          end else begin
            Brush.Color:=FieldColors[F AND UNMARK];
            if (F AND MARKED)<>0
              then FillRect(Rect(BX-MarkOffX,BY+MarkOffY,BX+BWidth-MarkOffX,BY+BHeight+MarkOffY))
              else FillRect(Rect(BX,BY,BX+BWidth,BY+BHeight));
          end;
        if (F AND MARKED)=0 then begin
          G:=Field[X+1,Y];
          if ((G AND MARKED)=0) AND (F<>G) then begin
            MoveTo(BX+BWidth-1,BY-1); LineTo(BX+BWidth-1,BY+BHeight);
          end;
          G:=Field[X,Y+1];
          if ((G AND MARKED)=0) AND (F<>G) then begin
            MoveTo(BX-1,BY+BHeight-1); LineTo(BX+BWidth,BY+BHeight-1);
          end;
        end;
      inc(BX,BWidth); end;
    inc(BY,BHeight); end;
    MoveTo(Width*BWidth-1,0);
    LineTo(Width*BWidth-1,Height*BHeight-1);
    LineTo(-1,Height*BHeight-1);
  end;
  Canvas.Draw(0,0,Cache);
end;

function LoadFieldBitmaps(const FileName:string):integer;
var BMP:TBitmap;
    i,Y:integer;
begin
  Result:=0; UseBitmaps:=false;
  if FileName='' then exit;
  try
    BMP:=TBitmap.Create;
    BMP.LoadFromFile(FileName);
    Result:=BMP.Width;
  except
    exit;
  end;
  Y:=0;
  for i:=1 to MAXCOLORS do begin
    if Assigned(FieldBitmaps[i]) then FieldBitmaps[i].Free;
    FieldBitmaps[i]:=TBitmap.Create;
    with FieldBitmaps[i] do begin
      Width:=Result;
      Height:=Result;
      PixelFormat:=pfDevice;
      Canvas.Draw(0,Y,BMP);
    end;
    dec(Y,Result);
  end;
  BMP.Free;
  UseBitmaps:=true;
end;

function LoadBackgroundImage(const FileName:string; IWidth,IHeight:integer):boolean;
var JPEG:TJPEGImage;
    BMP:TBitmap;
    NX,NY:integer;
begin
  Result:=false;
  try
    if Assigned(FieldBitmaps[0]) then FieldBitmaps[0].Free;
    if FileName='' then exit;
    JPEG:=TJPEGImage.Create;
    JPEG.LoadFromFile(FileName);
    NX:=JPEG.Width;
    NY:=JPEG.Width*IHeight DIV IWidth;
    if NY>JPEG.Height then begin
      NX:=JPEG.Height*IWidth DIV IHeight;
      NY:=JPEG.Height;
    end;
    BMP:=TBitmap.Create;
    BMP.Width:=NX;
    BMP.Height:=NY;
    BMP.PixelFormat:=pf24bit;
    BMP.Canvas.Draw((NX-JPEG.Width) DIV 2,(NY-JPEG.Height) DIV 2,JPEG);
    JPEG.Free;
    FieldBitmaps[0]:=TBitmap.Create;
    FieldBitmaps[0].Width:=IWidth;
    FieldBitmaps[0].Height:=IHeight;
    Scale(BMP,FieldBitmaps[0]);
    BMP.Free;
    FieldBitmaps[0].PixelFormat:=pfDevice;
  except exit; end;
  Result:=true;
end;

var i:integer;
initialization
  Cache:=nil;
  FieldColors:=DefaultFieldColors;
  for i:=0 to MAXCOLORS do
    FieldBitmaps[i]:=nil;
end.
