unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Field, FieldPaint, ExtCtrls, StdCtrls, ComCtrls, ToolWin,
  ImgList;

type
  TMainForm = class(TForm)
    PBox: TPanel;
    Box: TPaintBox;
    Toolz: TToolBar;
    Status: TStatusBar;
    BNew: TToolButton;
    BUndo: TToolButton;
    BRedo: TToolButton;
    BSetup: TToolButton;
    Sep1: TToolButton;
    Images: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure BoxPaint(Sender: TObject);
    procedure BoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BNewClick(Sender: TObject);
    procedure BUndoClick(Sender: TObject);
    procedure BRedoClick(Sender: TObject);
    procedure BSetupClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    Field:TField;
    MaxQuota:integer;
    PropX,PropY:integer;
    procedure PaintField(XO,YO:integer);
    function FindProp:boolean;
    procedure CheckGameOver;
  public
    { Public declarations }
    FWidth,FHeight:integer;
    BWidth,BHeight:integer;
    FColors:integer;
    AnimSpeed:integer;
    BitmapName:string;
    Background:string;
    procedure ResizeField;
  end;

var
  MainForm: TMainForm;

implementation

uses Setup,LoadStore;

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  randomize;
  Field:=TField.Create;
  FWidth:=10; FHeight:=16;
  BWidth:=32; BHeight:=32;
  FColors:=5;
  AnimSpeed:=3;
  BitmapName:='Default32.bmp';
  Background:='DefaultBack.jpg';
  LoadIt;
  ResizeField;
end;

procedure TMainForm.BoxPaint(Sender: TObject);
begin
  Box.Canvas.Draw(0,0,Cache);
end;

procedure TMainForm.PaintField(XO,YO:integer);
begin
  DrawField(Field,Box.Canvas,BWidth,BHeight,XO,YO);
end;

procedure TMainForm.BoxMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var Quota,i:integer;
begin
  if Button=mbLeft then begin
    Quota:=Field.MarkAdjacent(X DIV BWidth,Y DIV BHeight);
    if Quota>1 then begin
      if Quota>MaxQuota then MaxQuota:=Quota;
      Status.SimpleText:='Zug Nr. '+IntToStr(Field.UndoCount+1)+' | '+
                         IntToStr(Quota)+' Steine | Rekord: '+IntToStr(MaxQuota);
      Field.RemoveMarked;
      PaintField(0,0);
      while Field.MarkFalling do begin
        for i:=1 to BHeight do
          if (i MOD AnimSpeed)=0 then
            PaintField(0,i);
        Field.FallMarked;
      end;
      while Field.MarkSliding do begin
        for i:=1 to BWidth do
          if (i MOD AnimSpeed)=0 then
            PaintField(i,0);
        Field.SlideMarked;
      end;
      PaintField(0,0);
      Field.AddUndo;
      CheckGameOver;
    end else
      Field.RemoveMarks;
  end;
end;

procedure TMainForm.BNewClick(Sender: TObject);
begin
  Field.InitRandomly(FWidth,FHeight,FColors);
  PaintField(0,0);
  MaxQuota:=0;
  Status.SimpleText:='Neues Spiel '+IntToStr(FWidth)+'x'+IntToStr(FHeight)+
                     ' gestartet.';
  CheckGameOver;
end;

procedure TMainForm.ResizeField;
var BM:integer;
begin
  if not LoadBackgroundImage(Background,FWidth*BWidth,FHeight*BHeight)
    then Background:='';
  BM:=LoadFieldBitmaps(BitmapName);
  if BM>0 then begin
    BWidth:=BM; BHeight:=BM;
  end;
  Box.Width:=FWidth*BWidth;
  Box.Height:=FHeight*BHeight;
  ClientWidth:=FWidth*BWidth+5;
  ClientHeight:=FHeight*BHeight+5+Toolz.Height+Status.Height;
  BNewClick(nil);
end;

procedure TMainForm.BUndoClick(Sender: TObject);
begin
  if not Field.Undo then exit;
  PaintField(0,0);
end;

procedure TMainForm.BRedoClick(Sender: TObject);
begin
  if not Field.Redo then exit;
  PaintField(0,0);
end;

function TMainForm.FindProp:boolean;
var X,Y:integer;
begin
  for Y:=0 to (Field.Height-1) do
    for X:=0 to (Field.Width-1) do
      if Field.MarkAdjacent(X,Y)>1 then begin
        PropX:=X; PropY:=Y;
        Result:=true; exit;
      end;
  PropX:=-1; PropY:=-1;
  Result:=false;
end;

procedure TMainForm.CheckGameOver;
var X,Y,C:integer;
begin
  if FindProp then exit;
  Field.RemoveMarks; C:=0;
  for Y:=0 to (Field.Height-1) do
    for X:=0 to (Field.Width-1) do
      if Field[X,Y]<>0 then
        inc(C);
  Status.SimpleText:='Game Over  '+IntToStr(C)+' Steine brig';
end;

procedure TMainForm.BSetupClick(Sender: TObject);
var i:integer;
begin
  with SetupForm do if ShowModal=mrOK then begin
    FWidth:=CFWidth.Position;
    FHeight:=CFHeight.Position;
    BWidth:=CBWidth.Position;
    BHeight:=CBHeight.Position;
    FColors:=CFColors.Position;
    AnimSpeed:=CAnimSpeed.Position;
    if CBitmap.Checked
      then BitmapName:=EBitmap.Text
      else BitmapName:='';
    if CBackground.Checked
      then Background:=EBackground.Text
      else Background:='';
    for i:=0 to MAXCOLORS do
      FieldColors[i]:=PColor[i].Color;
    ResizeField;
  end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  StoreIt;   
end;

end.

