unit Setup;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls, ExtDlgs, FieldPaint;

type
  TSetupForm = class(TForm)
    BOK: TButton;
    BCancel: TButton;
    GFB: TGroupBox;
    LFSize: TLabel;
    EFWidth: TEdit;
    CFWidth: TUpDown;
    LFx: TLabel;
    EFHeight: TEdit;
    CFHeight: TUpDown;
    LBSize: TLabel;
    EBWidth: TEdit;
    CBWidth: TUpDown;
    LBx: TLabel;
    EBHeight: TEdit;
    CBHeight: TUpDown;
    LFColors: TLabel;
    EFColors: TEdit;
    CFColors: TUpDown;
    LPixSize: TLabel;
    GAnim: TGroupBox;
    LAnimSpeed: TLabel;
    EAnimSpeed: TEdit;
    CAnimSpeed: TUpDown;
    GBitmap: TGroupBox;
    CBitmap: TCheckBox;
    EBitmap: TEdit;
    BBitmap: TButton;
    OpenBitmap: TOpenPictureDialog;
    CBackground: TCheckBox;
    GBackground: TGroupBox;
    EBackground: TEdit;
    BBackground: TButton;
    OpenBackground: TOpenPictureDialog;
    GColors: TGroupBox;
    ColorPicker: TColorDialog;
    LCommercial: TLabel;
    procedure FormShow(Sender: TObject);
    procedure SizeChange(Sender: TObject; Button: TUDBtnType);
    procedure FormCreate(Sender: TObject);
    procedure CBitmapClick(Sender: TObject);
    procedure BBitmapClick(Sender: TObject);
    procedure CBackgroundClick(Sender: TObject);
    procedure BBackgroundClick(Sender: TObject);
    procedure SomethingElseChanged(Sender: TObject; Button: TUDBtnType);
  private
    { Private declarations }
    PAXX,PAXY:integer;
    procedure PColorClick(Sender: TObject);
  public
    { Public declarations }
    PColor:array[0..MAXCOLORS]of TPanel;
  end;

var
  SetupForm: TSetupForm;

function ShortenPath(const FullPath:string):string;

implementation
uses Main;

{$R *.dfm}

const C_XOFFSET=8;
      C_YOFFSET=16;
      C_XSIZE=20;
      C_YSIZE=20;
      C_QPERLINE=5;

function ShortenPath(const FullPath:string):string;
var CPath,FPath:string;
begin
  CPath:=IncludeTrailingPathDelimiter(ExpandUNCFileName(GetCurrentDir));
  FPath:=ExpandUNCFileName(FullPath);
  if LowerCase(CPath)=LowerCase(copy(FPath,1,length(CPath)))
    then Result:=copy(FPath,length(CPath)+1,length(FPath)-length(CPath))
    else Result:=FullPath;
end;

procedure TSetupForm.FormCreate(Sender: TObject);
var i,X,Y:integer;
begin
  CFColors.Max:=MAXCOLORS;
  Y:=C_YOFFSET-C_YSIZE; X:=0;
  for i:=0 to MAXCOLORS do begin
    if (i MOD C_QPERLINE)=0 then
      begin X:=C_XOFFSET; inc(Y,C_YSIZE); end;
    PColor[i]:=TPanel.Create(Self);
    with PColor[i] do begin
      Parent:=GColors;
      Left:=X; Top:=Y;
      Width:=C_XSIZE; Height:=C_YSIZE;
      BevelOuter:=bvLowered;
      Cursor:=crHandPoint;
      ShowHint:=true; ParentShowHint:=false;
      if i=0 then Hint:='Hintergrundfarbe'
             else Hint:='Farbe Nr. '+IntToStr(i);
      OnClick:=PColorClick;
      Tag:=i;
    end;
    inc(X,C_XSIZE);
  end;
end;

procedure TSetupForm.FormShow(Sender: TObject);
var i:integer;
begin
  with MainForm do begin
    PAXX:=Width-FWidth*BWidth;
    PAXY:=Height-FHeight*BHeight;
    CFWidth.Position:=FWidth;
    CFHeight.Position:=FHeight;
    CBWidth.Position:=BWidth;
    CBHeight.Position:=BHeight;
    CFColors.Position:=FColors;
    CAnimSpeed.Position:=AnimSpeed;
    CBitmap.Checked:=UseBitmaps;
    EBitmap.Text:=BitmapName;
    CBackground.Checked:=Assigned(FieldBitmaps[0]);
    EBackground.Text:=Background;
    BOK.Enabled:=false;
    BCancel.Enabled:=true;
  end;
  for i:=0 to MAXCOLORS do
    PColor[i].Color:=FieldColors[i];
  CBitmapClick(nil);
  CBackgroundClick(nil);
  SizeChange(nil,btNext);
end;

procedure TSetupForm.SizeChange(Sender: TObject; Button: TUDBtnType);
begin
  LPixSize.Caption:=
    IntToStr(CFWidth.Position*CBWidth.Position+PAXX)+'x'+
    IntToStr(CFHeight.Position*CBHeight.Position+PAXY)+' Pixel';
  if Assigned(Sender) then BOK.Enabled:=true;
end;

procedure TSetupForm.CBitmapClick(Sender: TObject);
var q:boolean;
begin
  q:=CBitmap.Checked;
  EBitmap.Enabled:=q;
  BBitmap.Enabled:=q;
  EBWidth.Enabled:=not(q);
  CBWidth.Enabled:=not(q);
  EBHeight.Enabled:=not(q);
  CBHeight.Enabled:=not(q);
  if Assigned(Sender) then
    BOK.Enabled:=true;
end;

procedure TSetupForm.BBitmapClick(Sender: TObject);
var BM:integer;
begin
  if not OpenBitmap.Execute then exit;
  EBitmap.Text:=ShortenPath(OpenBitmap.FileName);
  BOK.Enabled:=true;
  BCancel.Enabled:=false;
  BM:=LoadFieldBitmaps(EBitmap.Text);
  if BM=0 then begin
    CBitmap.Checked:=false;
    CBitmapClick(nil);
  end else begin
    CBWidth.Position:=BM;
    CBHeight.Position:=BM;
  end;
end;

procedure TSetupForm.CBackgroundClick(Sender: TObject);
var q:boolean;
begin
  q:=CBackground.Checked;
  EBackground.Enabled:=q;
  BBackground.Enabled:=q;
  if Assigned(Sender) then
    BOK.Enabled:=true;
end;

procedure TSetupForm.BBackgroundClick(Sender: TObject);
begin
  if not OpenBackground.Execute then exit;
  EBackground.Text:=ShortenPath(OpenBackground.FileName);
  BOK.Enabled:=true;
end;

procedure TSetupForm.PColorClick(Sender: TObject);
begin
  with (Sender as TPanel) do begin
    ColorPicker.Color:=Color;
    if not ColorPicker.Execute then exit;
    Color:=ColorPicker.Color;
    BOK.Enabled:=true;
  end;
end;

procedure TSetupForm.SomethingElseChanged(Sender: TObject;
  Button: TUDBtnType);
begin
  if Assigned(Sender) then
    BOK.Enabled:=true;
end;

end.
